@@ -21,8 +21,6 @@
  */
 
 #include "sysdeps.h"
-#include <va/va.h>
-#include <va/va_enc_vp8.h>
 #include <gst/base/gstbitwriter.h>
 #include <gst/codecparsers/gstvp8parser.h>
 #include "gstvaapicompat.h"
@@ -40,7 +38,8 @@
 /* Supported set of VA rate controls, within this implementation */
 #define SUPPORTED_RATECONTROLS                  \
   (GST_VAAPI_RATECONTROL_MASK (CQP) |           \
-   GST_VAAPI_RATECONTROL_MASK (CBR))
+   GST_VAAPI_RATECONTROL_MASK (CBR) |           \
+   GST_VAAPI_RATECONTROL_MASK (VBR))
 
 /* Supported set of tuning options, within this implementation */
 #define SUPPORTED_TUNE_OPTIONS \
@@ -58,9 +57,6 @@
 /* --- VP8 Encoder                                                      --- */
 /* ------------------------------------------------------------------------- */
 
-#define GST_VAAPI_ENCODER_VP8_CAST(encoder) \
-    ((GstVaapiEncoderVP8 *)(encoder))
-
 struct _GstVaapiEncoderVP8
 {
   GstVaapiEncoder parent_instance;
@@ -112,7 +108,8 @@ ensure_hw_profile (GstVaapiEncoderVP8 * encoder)
   /* ERRORS */
 error_unsupported_profile:
   {
-    GST_ERROR ("unsupported HW profile (0x%08x)", encoder->profile);
+    GST_ERROR ("unsupported HW profile %s",
+        gst_vaapi_profile_get_name (encoder->profile));
     return FALSE;
   }
 }
@@ -125,6 +122,7 @@ ensure_bitrate (GstVaapiEncoderVP8 * encoder)
   /* Default compression: 64 bits per macroblock  */
   switch (GST_VAAPI_ENCODER_RATE_CONTROL (encoder)) {
     case GST_VAAPI_RATECONTROL_CBR:
+    case GST_VAAPI_RATECONTROL_VBR:
       if (!base_encoder->bitrate) {
         base_encoder->bitrate =
             gst_util_uint64_scale (GST_VAAPI_ENCODER_WIDTH (encoder) *
@@ -144,7 +142,7 @@ ensure_bitrate (GstVaapiEncoderVP8 * encoder)
 static GstVaapiEncoderStatus
 set_context_info (GstVaapiEncoder * base_encoder)
 {
-  GstVaapiEncoderVP8 *encoder = GST_VAAPI_ENCODER_VP8_CAST (base_encoder);
+  GstVaapiEncoderVP8 *encoder = GST_VAAPI_ENCODER_VP8 (base_encoder);
   GstVideoInfo *const vip = GST_VAAPI_ENCODER_VIDEO_INFO (encoder);
 
   /* Maximum sizes for common headers (in bytes) */
@@ -220,7 +218,8 @@ fill_sequence (GstVaapiEncoderVP8 * encoder, GstVaapiEncSequence * sequence)
   seq_param->frame_width = GST_VAAPI_ENCODER_WIDTH (encoder);
   seq_param->frame_height = GST_VAAPI_ENCODER_HEIGHT (encoder);
 
-  if (GST_VAAPI_ENCODER_RATE_CONTROL (encoder) & GST_VAAPI_RATECONTROL_CBR)
+  if (GST_VAAPI_ENCODER_RATE_CONTROL (encoder) == GST_VAAPI_RATECONTROL_CBR ||
+      GST_VAAPI_ENCODER_RATE_CONTROL (encoder) == GST_VAAPI_RATECONTROL_VBR)
     seq_param->bits_per_second = base_encoder->bitrate * 1000;
 
   seq_param->intra_period = base_encoder->keyframe_period;
@@ -257,61 +256,39 @@ error:
   }
 }
 
+static gboolean
+ensure_control_rate_params (GstVaapiEncoderVP8 * encoder)
+{
+  GstVaapiEncoder *const base_encoder = GST_VAAPI_ENCODER_CAST (encoder);
+
+  if (GST_VAAPI_ENCODER_RATE_CONTROL (encoder) == GST_VAAPI_RATECONTROL_CQP)
+    return TRUE;
+
+  /* RateControl params */
+  GST_VAAPI_ENCODER_VA_RATE_CONTROL (encoder).initial_qp = encoder->yac_qi;
+  GST_VAAPI_ENCODER_VA_RATE_CONTROL (encoder).min_qp = 1;
+
+  /* *INDENT-OFF* */
+  /* HRD params */
+  GST_VAAPI_ENCODER_VA_HRD (encoder) = (VAEncMiscParameterHRD) {
+    .buffer_size = base_encoder->bitrate * 1000 * 2,
+    .initial_buffer_fullness = base_encoder->bitrate * 1000,
+  };
+  /* *INDENT-ON* */
+
+  return TRUE;
+}
+
 static gboolean
 ensure_misc_params (GstVaapiEncoderVP8 * encoder, GstVaapiEncPicture * picture)
 {
   GstVaapiEncoder *const base_encoder = GST_VAAPI_ENCODER_CAST (encoder);
-  GstVaapiEncMiscParam *misc;
 
-  if (GST_VAAPI_ENCODER_RATE_CONTROL (encoder) != GST_VAAPI_RATECONTROL_CBR)
-    return TRUE;
-
-  misc = GST_VAAPI_ENC_MISC_PARAM_NEW (FrameRate, encoder);
-  if (!misc)
+  if (!gst_vaapi_encoder_ensure_param_quality_level (base_encoder, picture))
     return FALSE;
-  {
-    VAEncMiscParameterFrameRate *param = misc->data;
-    param->framerate =
-        GST_VAAPI_ENCODER_FPS_N (encoder) / GST_VAAPI_ENCODER_FPS_D (encoder);
-  }
-  gst_vaapi_enc_picture_add_misc_param (picture, misc);
-  gst_vaapi_codec_object_replace (&misc, NULL);
 
-  misc = GST_VAAPI_ENC_MISC_PARAM_NEW (HRD, encoder);
-  if (!misc)
+  if (!gst_vaapi_encoder_ensure_param_control_rate (base_encoder, picture))
     return FALSE;
-  {
-    VAEncMiscParameterHRD *hrd = misc->data;
-    if (base_encoder->bitrate > 0) {
-      hrd->buffer_size = base_encoder->bitrate * 1000 * 2;
-      hrd->initial_buffer_fullness = base_encoder->bitrate * 1000;
-    } else {
-      hrd->buffer_size = 0;
-      hrd->initial_buffer_fullness = 0;
-    }
-  }
-
-  gst_vaapi_enc_picture_add_misc_param (picture, misc);
-  gst_vaapi_codec_object_replace (&misc, NULL);
-
-  /* RateControl params */
-  misc = GST_VAAPI_ENC_MISC_PARAM_NEW (RateControl, encoder);
-  if (!misc)
-    return FALSE;
-  {
-    VAEncMiscParameterRateControl *rate_control;
-    rate_control = misc->data;
-    rate_control->bits_per_second = base_encoder->bitrate * 1000;
-    rate_control->target_percentage = 70;
-    /* CPB (Coded picture buffer) length in milliseconds, which could
-     * be provided as a property */
-    rate_control->window_size = 500;
-    rate_control->initial_qp = encoder->yac_qi;
-    rate_control->min_qp = 1;
-  }
-
-  gst_vaapi_enc_picture_add_misc_param (picture, misc);
-  gst_vaapi_codec_object_replace (&misc, NULL);
 
   return TRUE;
 }
@@ -429,7 +406,7 @@ static GstVaapiEncoderStatus
 gst_vaapi_encoder_vp8_encode (GstVaapiEncoder * base_encoder,
     GstVaapiEncPicture * picture, GstVaapiCodedBufferProxy * codedbuf)
 {
-  GstVaapiEncoderVP8 *const encoder = GST_VAAPI_ENCODER_VP8_CAST (base_encoder);
+  GstVaapiEncoderVP8 *const encoder = GST_VAAPI_ENCODER_VP8 (base_encoder);
   GstVaapiEncoderStatus ret = GST_VAAPI_ENCODER_STATUS_ERROR_UNKNOWN;
   GstVaapiSurfaceProxy *reconstruct = NULL;
 
@@ -475,7 +452,7 @@ static GstVaapiEncoderStatus
 gst_vaapi_encoder_vp8_reordering (GstVaapiEncoder * base_encoder,
     GstVideoCodecFrame * frame, GstVaapiEncPicture ** output)
 {
-  GstVaapiEncoderVP8 *const encoder = GST_VAAPI_ENCODER_VP8_CAST (base_encoder);
+  GstVaapiEncoderVP8 *const encoder = GST_VAAPI_ENCODER_VP8 (base_encoder);
   GstVaapiEncPicture *picture = NULL;
   GstVaapiEncoderStatus status = GST_VAAPI_ENCODER_STATUS_SUCCESS;
 
@@ -508,7 +485,7 @@ gst_vaapi_encoder_vp8_reordering (GstVaapiEncoder * base_encoder,
 static GstVaapiEncoderStatus
 gst_vaapi_encoder_vp8_reconfigure (GstVaapiEncoder * base_encoder)
 {
-  GstVaapiEncoderVP8 *const encoder = GST_VAAPI_ENCODER_VP8_CAST (base_encoder);
+  GstVaapiEncoderVP8 *const encoder = GST_VAAPI_ENCODER_VP8 (base_encoder);
   GstVaapiEncoderStatus status;
 
   status = ensure_profile (encoder);
@@ -518,6 +495,7 @@ gst_vaapi_encoder_vp8_reconfigure (GstVaapiEncoder * base_encoder)
   if (!ensure_bitrate (encoder))
     goto error;
 
+  ensure_control_rate_params (encoder);
   return set_context_info (base_encoder);
 
   /* ERRORS */
@@ -530,7 +508,7 @@ error:
 static gboolean
 gst_vaapi_encoder_vp8_init (GstVaapiEncoder * base_encoder)
 {
-  GstVaapiEncoderVP8 *const encoder = GST_VAAPI_ENCODER_VP8_CAST (base_encoder);
+  GstVaapiEncoderVP8 *const encoder = GST_VAAPI_ENCODER_VP8 (base_encoder);
 
   encoder->frame_num = 0;
   encoder->last_ref = NULL;
@@ -543,7 +521,7 @@ gst_vaapi_encoder_vp8_init (GstVaapiEncoder * base_encoder)
 static void
 gst_vaapi_encoder_vp8_finalize (GstVaapiEncoder * base_encoder)
 {
-  GstVaapiEncoderVP8 *const encoder = GST_VAAPI_ENCODER_VP8_CAST (base_encoder);
+  GstVaapiEncoderVP8 *const encoder = GST_VAAPI_ENCODER_VP8 (base_encoder);
   clear_references (encoder);
 }
 
@@ -551,7 +529,7 @@ static GstVaapiEncoderStatus
 gst_vaapi_encoder_vp8_set_property (GstVaapiEncoder * base_encoder,
     gint prop_id, const GValue * value)
 {
-  GstVaapiEncoderVP8 *const encoder = GST_VAAPI_ENCODER_VP8_CAST (base_encoder);
+  GstVaapiEncoderVP8 *const encoder = GST_VAAPI_ENCODER_VP8 (base_encoder);
 
   switch (prop_id) {
     case GST_VAAPI_ENCODER_VP8_PROP_LOOP_FILTER_LEVEL:
