@@ -66,6 +66,7 @@ typedef enum
   GST_VAAPI_CAPS_FEATURE_NOT_NEGOTIATED,
   GST_VAAPI_CAPS_FEATURE_SYSTEM_MEMORY,
   GST_VAAPI_CAPS_FEATURE_GL_TEXTURE_UPLOAD_META,
+  GST_VAAPI_CAPS_FEATURE_DMABUF,
   GST_VAAPI_CAPS_FEATURE_VAAPI_SURFACE,
 } GstVaapiCapsFeature;
 
@@ -110,6 +111,10 @@ gst_vaapi_caps_feature_contains (const GstCaps * caps,
     GST_VIDEO_CAPS_MAKE_WITH_FEATURES(					\
         GST_CAPS_FEATURE_META_GST_VIDEO_GL_TEXTURE_UPLOAD_META, "{ RGBA, BGRA }")
 
+#define GST_VAAPI_MAKE_DMABUF_CAPS					\
+    GST_VIDEO_CAPS_MAKE_WITH_FEATURES(					\
+        GST_CAPS_FEATURE_MEMORY_DMABUF, "{ I420, YV12, RGBA }")
+
 G_GNUC_INTERNAL
 gboolean
 gst_caps_set_interlaced (GstCaps * caps, GstVideoInfo * vip);
