@@ -392,6 +392,34 @@ error_create_buffer:
   }
 }
 
+static inline GstBuffer *
+create_output_dump_buffer (GstVaapiPostproc * postproc)
+{
+  GstVaapiPluginBase *const plugin = GST_VAAPI_PLUGIN_BASE (postproc);
+
+  return gst_buffer_new_allocate (plugin->other_srcpad_allocator,
+      GST_VIDEO_INFO_SIZE (&plugin->srcpad_info),
+      &plugin->other_allocator_params);
+}
+
+static void
+copy_metadata (GstVaapiPostproc * postproc, GstBuffer * outbuf,
+    GstBuffer * inbuf)
+{
+  GstBaseTransformClass *bclass = GST_BASE_TRANSFORM_GET_CLASS (postproc);
+  GstBaseTransform *trans = GST_BASE_TRANSFORM (postproc);
+
+  if (inbuf == outbuf)
+    return;
+  if (!bclass->copy_metadata)
+    return;
+  if (!bclass->copy_metadata (trans, inbuf, outbuf)) {
+    /* something failed, post a warning */
+    GST_ELEMENT_WARNING (trans, STREAM, NOT_IMPLEMENTED,
+        ("could not copy metadata"), (NULL));
+  }
+}
+
 static gboolean
 append_output_buffer_metadata (GstVaapiPostproc * postproc, GstBuffer * outbuf,
     GstBuffer * inbuf, guint flags)
@@ -401,16 +429,7 @@ append_output_buffer_metadata (GstVaapiPostproc * postproc, GstBuffer * outbuf,
 
   gst_buffer_copy_into (outbuf, inbuf, flags | GST_BUFFER_COPY_FLAGS, 0, -1);
 
-  /* GstVideoCropMeta */
-  if (!postproc->use_vpp) {
-    GstVideoCropMeta *const crop_meta = gst_buffer_get_video_crop_meta (inbuf);
-    if (crop_meta) {
-      GstVideoCropMeta *const out_crop_meta =
-          gst_buffer_add_video_crop_meta (outbuf);
-      if (out_crop_meta)
-        *out_crop_meta = *crop_meta;
-    }
-  }
+  copy_metadata (postproc, outbuf, inbuf);
 
   /* GstVaapiVideoMeta */
   inbuf_meta = gst_buffer_get_vaapi_video_meta (inbuf);
@@ -600,6 +619,31 @@ gst_vaapipostproc_set_passthrough (GstBaseTransform * trans)
       && !filter_updated);
 }
 
+static gboolean
+replace_to_dumb_buffer_if_required (GstVaapiPostproc * postproc,
+    GstBuffer ** fieldbuf)
+{
+  GstVaapiPluginBase *const plugin = GST_VAAPI_PLUGIN_BASE (postproc);
+  GstBuffer *newbuf;
+
+  if (!GST_VAAPI_PLUGIN_BASE_COPY_OUTPUT_FRAME (postproc))
+    return TRUE;
+
+  newbuf = create_output_dump_buffer (postproc);
+  if (!newbuf)
+    return FALSE;
+
+  if (!gst_vaapi_plugin_copy_va_buffer (plugin, *fieldbuf, newbuf)) {
+    gst_buffer_unref (newbuf);
+    return FALSE;
+  }
+
+  gst_buffer_replace (fieldbuf, newbuf);
+  gst_buffer_unref (newbuf);
+
+  return TRUE;
+}
+
 static GstFlowReturn
 gst_vaapipostproc_process_vpp (GstBaseTransform * trans, GstBuffer * inbuf,
     GstBuffer * outbuf)
@@ -615,7 +659,7 @@ gst_vaapipostproc_process_vpp (GstBaseTransform * trans, GstBuffer * inbuf,
   GstBuffer *fieldbuf;
   GstVaapiDeinterlaceMethod deint_method;
   guint flags, deint_flags;
-  gboolean tff, deint, deint_refs, deint_changed;
+  gboolean tff, deint, deint_refs, deint_changed, discont;
   const GstVideoCropMeta *crop_meta;
   GstVaapiRectangle *crop_rect = NULL;
   GstVaapiRectangle tmp_rect;
@@ -639,6 +683,7 @@ gst_vaapipostproc_process_vpp (GstBaseTransform * trans, GstBuffer * inbuf,
 
   timestamp = GST_BUFFER_TIMESTAMP (inbuf);
   tff = GST_BUFFER_FLAG_IS_SET (inbuf, GST_VIDEO_BUFFER_FLAG_TFF);
+  discont = GST_BUFFER_FLAG_IS_SET (inbuf, GST_BUFFER_FLAG_DISCONT);
   deint = should_deinterlace_buffer (postproc, inbuf);
 
   /* Drop references if deinterlacing conditions changed */
@@ -719,8 +764,17 @@ gst_vaapipostproc_process_vpp (GstBaseTransform * trans, GstBuffer * inbuf,
     if (status != GST_VAAPI_FILTER_STATUS_SUCCESS)
       goto error_process_vpp;
 
+    copy_metadata (postproc, fieldbuf, inbuf);
     GST_BUFFER_TIMESTAMP (fieldbuf) = timestamp;
     GST_BUFFER_DURATION (fieldbuf) = postproc->field_duration;
+    if (discont) {
+      GST_BUFFER_FLAG_SET (fieldbuf, GST_BUFFER_FLAG_DISCONT);
+      discont = FALSE;
+    }
+
+    if (!replace_to_dumb_buffer_if_required (postproc, &fieldbuf))
+      goto error_copy_buffer;
+
     ret = gst_pad_push (trans->srcpad, fieldbuf);
     if (ret != GST_FLOW_OK)
       goto error_push_buffer;
@@ -771,7 +825,12 @@ gst_vaapipostproc_process_vpp (GstBaseTransform * trans, GstBuffer * inbuf,
   else {
     GST_BUFFER_TIMESTAMP (outbuf) = timestamp + postproc->field_duration;
     GST_BUFFER_DURATION (outbuf) = postproc->field_duration;
+    if (discont) {
+      GST_BUFFER_FLAG_SET (fieldbuf, GST_BUFFER_FLAG_DISCONT);
+      discont = FALSE;
+    }
   }
+  copy_metadata (postproc, outbuf, inbuf);
 
   if (deint && deint_refs)
     ds_add_buffer (ds, inbuf);
@@ -814,6 +873,12 @@ error_process_vpp:
     gst_buffer_replace (&fieldbuf, NULL);
     return GST_FLOW_ERROR;
   }
+error_copy_buffer:
+  {
+    GST_ERROR_OBJECT (postproc, "failed to copy field buffer to dumb buffer");
+    gst_buffer_replace (&fieldbuf, NULL);
+    return GST_FLOW_ERROR;
+  }
 error_push_buffer:
   {
     GST_DEBUG_OBJECT (postproc, "failed to push output buffer: %s",
@@ -861,6 +926,10 @@ gst_vaapipostproc_process (GstBaseTransform * trans, GstBuffer * inbuf,
 
   GST_BUFFER_TIMESTAMP (fieldbuf) = timestamp;
   GST_BUFFER_DURATION (fieldbuf) = postproc->field_duration;
+
+  if (!replace_to_dumb_buffer_if_required (postproc, &fieldbuf))
+    goto error_copy_buffer;
+
   ret = gst_pad_push (trans->srcpad, fieldbuf);
   if (ret != GST_FLOW_OK)
     goto error_push_buffer;
@@ -891,6 +960,12 @@ error_create_buffer:
     GST_ERROR_OBJECT (postproc, "failed to create output buffer");
     return GST_FLOW_EOS;
   }
+error_copy_buffer:
+  {
+    GST_ERROR_OBJECT (postproc, "failed to copy field buffer to dumb buffer");
+    gst_buffer_replace (&fieldbuf, NULL);
+    return GST_FLOW_ERROR;
+  }
 error_push_buffer:
   {
     GST_DEBUG_OBJECT (postproc, "failed to push output buffer: %s",
@@ -1196,20 +1271,41 @@ gst_vaapipostproc_transform_size (GstBaseTransform * trans,
   return TRUE;
 }
 
+static gboolean
+gst_vaapipostproc_transform_meta (GstBaseTransform * trans, GstBuffer * outbuf,
+    GstMeta * meta, GstBuffer * inbuf)
+{
+  GstVaapiPostproc *const postproc = GST_VAAPIPOSTPROC (trans);
+
+  /* dont' GstVideoCropMeta if use_vpp */
+  if (meta->info->api == GST_VIDEO_CROP_META_API_TYPE && postproc->use_vpp)
+    return FALSE;
+  return TRUE;
+}
+
 static GstFlowReturn
 gst_vaapipostproc_transform (GstBaseTransform * trans, GstBuffer * inbuf,
     GstBuffer * outbuf)
 {
   GstVaapiPostproc *const postproc = GST_VAAPIPOSTPROC (trans);
-  GstBuffer *buf;
+  GstVaapiPluginBase *const plugin = GST_VAAPI_PLUGIN_BASE (postproc);
+  GstBuffer *buf, *sys_buf = NULL;
   GstFlowReturn ret;
 
-  ret =
-      gst_vaapi_plugin_base_get_input_buffer (GST_VAAPI_PLUGIN_BASE (postproc),
-      inbuf, &buf);
+  ret = gst_vaapi_plugin_base_get_input_buffer (plugin, inbuf, &buf);
   if (ret != GST_FLOW_OK)
     return GST_FLOW_ERROR;
 
+  if (GST_VAAPI_PLUGIN_BASE_COPY_OUTPUT_FRAME (trans)) {
+    GstBuffer *va_buf = create_output_buffer (postproc);
+    if (!va_buf) {
+      ret = GST_FLOW_ERROR;
+      goto done;
+    }
+    sys_buf = outbuf;
+    outbuf = va_buf;
+  }
+
   ret = GST_FLOW_NOT_SUPPORTED;
   if (postproc->flags) {
     /* Use VA/VPP extensions to process this frame */
@@ -1235,6 +1331,15 @@ gst_vaapipostproc_transform (GstBaseTransform * trans, GstBuffer * inbuf,
 
 done:
   gst_buffer_unref (buf);
+
+  if (sys_buf) {
+    if (!gst_vaapi_plugin_copy_va_buffer (plugin, outbuf, sys_buf))
+      return GST_FLOW_ERROR;
+
+    gst_buffer_unref (outbuf);
+    outbuf = sys_buf;
+  }
+
   return ret;
 }
 
@@ -1249,8 +1354,16 @@ gst_vaapipostproc_prepare_output_buffer (GstBaseTransform * trans,
     return GST_FLOW_OK;
   }
 
-  *outbuf_ptr = create_output_buffer (postproc);
-  return *outbuf_ptr ? GST_FLOW_OK : GST_FLOW_ERROR;
+  if (GST_VAAPI_PLUGIN_BASE_COPY_OUTPUT_FRAME (trans)) {
+    *outbuf_ptr = create_output_dump_buffer (postproc);
+  } else {
+    *outbuf_ptr = create_output_buffer (postproc);
+  }
+
+  if (!*outbuf_ptr)
+    return GST_FLOW_ERROR;
+
+  return GST_FLOW_OK;
 }
 
 static gboolean
@@ -1586,6 +1699,7 @@ gst_vaapipostproc_class_init (GstVaapiPostprocClass * klass)
   trans_class->fixate_caps = gst_vaapipostproc_fixate_caps;
   trans_class->transform_caps = gst_vaapipostproc_transform_caps;
   trans_class->transform_size = gst_vaapipostproc_transform_size;
+  trans_class->transform_meta = gst_vaapipostproc_transform_meta;
   trans_class->transform = gst_vaapipostproc_transform;
   trans_class->set_caps = gst_vaapipostproc_set_caps;
   trans_class->query = gst_vaapipostproc_query;
